# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Search dumper for the email field."""


from invenio_records.dumpers import SearchDumperExt


class EmailFieldDumperExt(SearchDumperExt):
    """Search dumper extension for the email field."""

    def __init__(self, field):
        """Constructor."""
        super().__init__()
        self.field = field
        self.hidden_field = f"{field}_hidden"

    def dump(self, record, data):
        """Dump the data."""
        email = data.pop(self.field, None)

        email_visible = record.preferences["email_visibility"]
        if email_visible == "public":
            data[self.field] = email
        data[self.hidden_field] = email

    def load(self, data, record_cls):
        """Load the data."""
        email = data.pop(self.hidden_field, None)
        data.setdefault(self.field, email)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 TU Wien.
# Copyright (C) 2022 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Users resource config."""

import marshmallow as ma
from flask_resources import HTTPJSONException, create_error_handler
from invenio_cache.errors import LockAcquireFailed
from invenio_i18n import lazy_gettext as _
from invenio_records_resources.resources import (
    RecordResourceConfig,
    SearchRequestArgsSchema,
)
from invenio_records_resources.resources.errors import ErrorHandlersMixin
from marshmallow import fields


class UsersSearchRequestArgsSchema(SearchRequestArgsSchema):
    """Add parameter to parse tags."""

    is_active = fields.Boolean()
    is_blocked = fields.Boolean()
    is_verified = fields.Boolean()


#
# Resource config
#
class UsersResourceConfig(RecordResourceConfig):
    """Users resource configuration."""

    blueprint_name = "users"
    url_prefix = "/users"
    routes = {
        "list": "",
        "search_all": "/all",
        "item": "/<id>",
        "item-avatar": "/<id>/avatar.svg",
        "approve": "/<id>/approve",
        "block": "/<id>/block",
        "restore": "/<id>/restore",
        "activate": "/<id>/activate",
        "deactivate": "/<id>/deactivate",
        "impersonate": "/<id>/impersonate",
    }

    request_view_args = {
        "id": ma.fields.Str(),
    }

    request_search_args = UsersSearchRequestArgsSchema

    error_handlers = {
        **ErrorHandlersMixin.error_handlers,
        LockAcquireFailed: create_error_handler(
            lambda e: (
                HTTPJSONException(
                    code=400,
                    description=_(
                        "User is locked due to concurrent running operation."
                    ),
                )
            )
        ),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
#
# Invenio-Vocabularies is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Users search facets definitions."""

from invenio_i18n import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet

domain = TermsFacet(
    field="domain",
    label=_("Domain"),
)

domain_status = TermsFacet(
    field="domaininfo.status",
    label=_("Domain status"),
    value_labels={
        1: _("New"),
        2: _("Moderated"),
        3: _("Verified"),
        4: _("Blocked"),
    },
)

affiliations = TermsFacet(
    field="profile.affiliations.keyword",
    label=_("Affiliations"),
)

status = TermsFacet(
    field="status",
    label=_("Account status"),
    value_labels={
        "new": _("New"),
        "verified": _("Verified"),
        "confirmed": _("Confirmed"),
        "blocked": _("Blocked"),
        "inactive": _("Inactive"),
    },
)

visibility = TermsFacet(
    field="visibility",
    label=_("Profile visibility"),
    value_labels={
        "hidden": _("Hidden"),
        "profile": _("Profile"),
        "full": _("Full"),
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Groups resource tests."""


def test_group_avatar(app, client, group, not_managed_group, user_pub):
    res = client.get(f"/groups/{not_managed_group.name}/avatar.svg")
    assert res.status_code == 403

    user_pub.login(client)

    # unmanaged group can be retrieved
    res = client.get(f"/groups/{not_managed_group.name}/avatar.svg")
    assert res.status_code == 200
    assert res.mimetype == "image/svg+xml"
    data = res.get_data()

    # managed group can *not* be retrieved
    res = client.get(f"/groups/{group.name}/avatar.svg")
    assert res.status_code == 403


# TODO: test conditional requests
# TODO: test caching headers
# TODO: test invalid identifiers



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
# Copyright (C) 2022 TU Wien.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Users service tasks."""

from celery import shared_task
from flask import current_app
from invenio_records_resources.services.uow import UnitOfWork
from invenio_records_resources.tasks import send_change_notifications
from invenio_search.engine import search
from werkzeug.local import LocalProxy

from invenio_users_resources.services.users.lock import ModerationMutex

from ...proxies import current_actions_registry, current_users_service
from ...records.api import UserAggregate

renewal_timeout = LocalProxy(
    lambda: current_app.config.get("USERS_RESOURCES_MODERATION_LOCK_RENEWAL_TIMEOUT")
)


@shared_task(ignore_result=True)
def reindex_users(user_ids):
    """Reindex the given user."""
    index = current_users_service.record_cls.index
    if current_users_service.indexer.exists(index):
        try:
            user_agg = {
                user_id: UserAggregate.get_record(user_id) for user_id in user_ids
            }
            current_users_service.indexer.bulk_index(user_ids)

            # trigger reindexing of related records
            send_change_notifications(
                "users",
                [
                    (
                        user_agg[user_id].id,
                        str(user_agg[user_id].id),
                        user_agg[user_id].revision_id,
                    )
                    for user_id in user_ids
                ],
            )
        except search.exceptions.ConflictError as e:
            current_app.logger.warn(f"Could not bulk-reindex users: {e}")


@shared_task(ignore_result=True)
def unindex_users(user_ids):
    """Delete the given user from the index."""
    index = current_users_service.record_cls.index
    if current_users_service.indexer.exists(index):
        try:
            current_users_service.indexer.bulk_delete(user_ids)
        except search.exceptions.ConflictError as e:
            current_app.logger.warn(f"Could not bulk-unindex users: {e}")


@shared_task(ignore_result=True, acks_late=True, retry=True)
def execute_moderation_actions(user_id=None, action=None):
    """Execute moderation actions.

    Callbacks share the same UOW to guarantee data consistency.
    If any callback fails, then the error is logged and the UOW rolledback.

    Why ``acks_late``:
     - in case the worker fails unexpectedly.
    Why ``retry``:
        - if a task fails, it can be retried afterwards.
    """
    actions = current_actions_registry.get(action, [])

    with ModerationMutex(user_id) as lock:
        lock.acquire_or_renew(renewal_timeout)

        # Create a uow that is shared by all the callables
        uow = UnitOfWork()
        try:
            for callback in actions:
                callback(user_id, uow=uow)
            # Commit the uow when all the callbacks succeeded
            uow.commit()
        except Exception as e:
            current_app.logger.warn(
                f"Could not execute action '{action}' for user: {e}"
            )
            # If a callback fails, rollback the operation and stop processing callbacks
            uow.rollback()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Records-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Domains service component."""

from invenio_accounts.models import DomainOrg
from invenio_db import db
from invenio_records_resources.services.records.components import ServiceComponent


class DomainComponent(ServiceComponent):
    """Service component for metadata."""

    def create(self, identity, data=None, record=None, errors=None, **kwargs):
        """Inject fields into the record."""
        # Note, DB model takes care of setting tld
        record.domain = data["domain"]
        record.status = data["status"]
        # Optional values
        record.flagged = data.get("flagged", False)
        record.flagged_source = data.get("flagged_source", "")
        record.category = data.get("category", None)
        self._handle_org(data, record)

    def update(self, identity, data=None, record=None, **kwargs):
        """Inject update fields into the domain."""
        # Main part of the validation happens in the schema hence here we just
        # pass on already validated properties.

        # Required values
        record.status = data["status"]
        # Optional values
        record.flagged = data.get("flagged", record.flagged)
        record.flagged_source = data.get("flagged_source", record.flagged_source)
        record.category = data.get("category", record.category)
        self._handle_org(data, record)

    def _handle_org(self, data, record):
        # Handle organisation
        if "org" in data:
            if data["org"] is None:
                record.org_id = None
            else:
                org = data["org"]
                obj = DomainOrg.query.filter_by(pid=org["pid"]).one_or_none()
                if obj is None:
                    with db.session.begin_nested():
                        obj = DomainOrg.create(
                            org["pid"], org["name"], json=org.get("props", None)
                        )
                record.org_id = obj.id
